package com.google.inject.internal;

import com.google.inject.Key;
import com.google.inject.internal.InjectorImpl.JitLimitation;
import com.google.inject.spi.Dependency;

import javax.inject.Provider;

import static com.google.common.base.Preconditions.checkNotNull;

/**
 * Delegates to a custom factory which is also bound in the injector.
 */
final class BoundProviderFactory<T> extends ProviderInternalFactory<T> implements CreationListener {

    final Key<? extends javax.inject.Provider<? extends T>> providerKey;
    private final ProvisionListenerStackCallback<T> provisionCallback;
    private final InjectorImpl injector;
    private InternalFactory<? extends javax.inject.Provider<? extends T>> providerFactory;

    BoundProviderFactory(
            InjectorImpl injector,
            Key<? extends javax.inject.Provider<? extends T>> providerKey,
            Object source,
            ProvisionListenerStackCallback<T> provisionCallback) {
        super(source);
        this.provisionCallback = checkNotNull(provisionCallback, "provisionCallback");
        this.injector = injector;
        this.providerKey = providerKey;
    }

    public void notify(Errors errors) {
        try {
            providerFactory = injector.getInternalFactory(providerKey, errors.withSource(source), JitLimitation.NEW_OR_EXISTING_JIT);
        } catch (ErrorsException e) {
            errors.merge(e.getErrors());
        }
    }

    public T get(Errors errors, InternalContext context, Dependency<?> dependency, boolean linked)
            throws ErrorsException {
        context.pushState(providerKey, source);
        try {
            errors = errors.withSource(providerKey);
            javax.inject.Provider<? extends T> provider = providerFactory.get(errors, context, dependency, true);
            return circularGet(provider, errors, context, dependency, provisionCallback);
        } finally {
            context.popState();
        }
    }

    @Override
    protected T provision(Provider<? extends T> provider, Errors errors, Dependency<?> dependency,
                          ConstructionContext<T> constructionContext) throws ErrorsException {
        try {
            return super.provision(provider, errors, dependency, constructionContext);
        } catch (RuntimeException userException) {
            throw errors.errorInProvider(userException).toException();
        }
    }

    @Override
    public String toString() {
        return providerKey.toString();
    }
}
