package com.google.inject.internal;

import com.google.common.base.MoreObjects;
import com.google.inject.Key;
import com.google.inject.internal.InjectorImpl.JitLimitation;
import com.google.inject.spi.Dependency;

/**
 * A placeholder which enables us to swap in the real factory once the injector is created.
 * Used for a linked binding, so that getting the linked binding returns the link's factory.
 */
final class FactoryProxy<T> implements InternalFactory<T>, CreationListener {

    private final InjectorImpl injector;
    private final Key<T> key;
    private final Key<? extends T> targetKey;
    private final Object source;

    private InternalFactory<? extends T> targetFactory;

    FactoryProxy(InjectorImpl injector, Key<T> key, Key<? extends T> targetKey, Object source) {
        this.injector = injector;
        this.key = key;
        this.targetKey = targetKey;
        this.source = source;
    }

    public void notify(final Errors errors) {
        try {
            targetFactory = injector.getInternalFactory(targetKey, errors.withSource(source), JitLimitation.NEW_OR_EXISTING_JIT);
        } catch (ErrorsException e) {
            errors.merge(e.getErrors());
        }
    }

    public T get(Errors errors, InternalContext context, Dependency<?> dependency, boolean linked)
            throws ErrorsException {
        context.pushState(targetKey, source);
        try {
            return targetFactory.get(errors.withSource(targetKey), context, dependency, true);
        } finally {
            context.popState();
        }
    }

    @Override
    public String toString() {
        return MoreObjects.toStringHelper(FactoryProxy.class)
                .add("key", key)
                .add("provider", targetFactory)
                .toString();
    }
}
