package com.google.inject.internal;

import com.google.inject.Provider;
import com.google.inject.spi.Dependency;

import static com.google.common.base.Preconditions.checkNotNull;

final class InternalFactoryToProviderAdapter<T> implements InternalFactory<T> {

    private final Provider<? extends T> provider;
    private final Object source;

    public InternalFactoryToProviderAdapter(Provider<? extends T> provider, Object source) {
        this.provider = checkNotNull(provider, "provider");
        this.source = checkNotNull(source, "source");
    }

    public T get(Errors errors, InternalContext context, Dependency<?> dependency, boolean linked)
            throws ErrorsException {
        // TODO(sameb): Does this need to push state into the context?
        try {
            return errors.checkForNull(provider.get(), source, dependency);
        } catch (RuntimeException userException) {
            throw errors.withSource(source).errorInProvider(userException).toException();
        }
    }

    @Override
    public String toString() {
        return provider.toString();
    }
}
