package com.google.inject.internal;

import com.google.inject.MembersInjector;
import com.google.inject.Provider;
import com.google.inject.spi.MembersInjectorLookup;
import com.google.inject.spi.ProviderLookup;

/**
 * Handles {@code Binder.getProvider} and {@code Binder.getMembersInjector(TypeLiteral)} commands.
 */
final class LookupProcessor extends AbstractProcessor {

    LookupProcessor(Errors errors) {
        super(errors);
    }

    @Override
    public <T> Boolean visit(MembersInjectorLookup<T> lookup) {
        try {
            MembersInjector<T> membersInjector
                    = injector.membersInjectorStore.get(lookup.getType(), errors);
            lookup.initializeDelegate(membersInjector);
        } catch (ErrorsException e) {
            errors.merge(e.getErrors()); // TODO: source
        }

        return true;
    }

    @Override
    public <T> Boolean visit(ProviderLookup<T> lookup) {
        // ensure the provider can be created
        try {
            Provider<T> provider = injector.getProviderOrThrow(lookup.getDependency(), errors);
            lookup.initializeDelegate(provider);
        } catch (ErrorsException e) {
            errors.merge(e.getErrors()); // TODO: source
        }

        return true;
    }
}
