package com.google.inject.internal;

import com.google.common.collect.Lists;

import java.util.List;

/**
 * Keeps track of creation listeners & uninitialized bindings,
 * so they can be processed after bindings are recorded.
 */
class ProcessedBindingData {

    private final List<CreationListener> creationListeners = Lists.newArrayList();
    private final List<Runnable> uninitializedBindings = Lists.newArrayList();

    void addCreationListener(CreationListener listener) {
        creationListeners.add(listener);
    }

    void addUninitializedBinding(Runnable runnable) {
        uninitializedBindings.add(runnable);
    }

    void initializeBindings() {
        for (Runnable initializer : uninitializedBindings) {
            initializer.run();
        }
    }

    void runCreationListeners(Errors errors) {
        for (CreationListener creationListener : creationListeners) {
            creationListener.notify(errors);
        }
    }

}
