package com.google.inject.name;

import java.lang.annotation.Annotation;

import static com.google.common.base.Preconditions.checkNotNull;

class NamedImpl implements Named {

    private final String value;

    public NamedImpl(String value) {
        this.value = checkNotNull(value, "name");
    }

    public String value() {
        return this.value;
    }

    public int hashCode() {
        // This is specified in java.lang.Annotation.
        return (127 * "value".hashCode()) ^ value.hashCode();
    }

    public boolean equals(Object o) {
        if (!(o instanceof Named)) {
            return false;
        }

        Named other = (Named) o;
        return value.equals(other.value());
    }

    public String toString() {
        return "@" + Named.class.getName() + "(value=" + value + ")";
    }

    public Class<? extends Annotation> annotationType() {
        return Named.class;
    }
}
