package com.google.inject.spi;

import com.google.inject.Binder;

import static com.google.common.base.Preconditions.checkNotNull;

/**
 * A request to disable circular proxies.
 */
public final class DisableCircularProxiesOption implements Element {
    private final Object source;

    DisableCircularProxiesOption(Object source) {
        this.source = checkNotNull(source, "source");
    }

    public Object getSource() {
        return source;
    }

    public void applyTo(Binder binder) {
        binder.withSource(getSource()).disableCircularProxies();
    }

    public <T> T acceptVisitor(ElementVisitor<T> visitor) {
        return visitor.visit(this);
    }
}
