package com.google.inject.spi;

import com.google.inject.Binding;
import com.google.inject.Key;
import com.google.inject.Provider;

/**
 * A binding to a {@link Provider} that delegates to the binding for the provided type. This binding
 * is used whenever a {@code Provider<T>} is injected (as opposed to injecting {@code T} directly).
 */
public interface ProviderBinding<T extends Provider<?>> extends Binding<T> {

    /**
     * Returns the key whose binding is used to {@link Provider#get provide instances}. That binding
     * can be retrieved from an injector using {@link com.google.inject.Injector#getBinding(Key)
     * Injector.getBinding(providedKey)}
     */
    Key<?> getProvidedKey();
}