package com.google.inject.spi;

import com.google.inject.Binding;

import java.util.Set;

/**
 * A binding to a provider instance. The provider's {@code get} method is invoked to resolve
 * injections.
 */
public interface ProviderInstanceBinding<T> extends Binding<T>, HasDependencies {

    /**
     * Returns the user-supplied, unscoped provider.
     */
    javax.inject.Provider<? extends T> getUserSuppliedProvider();

    /**
     * Returns the field and method injection points of the provider, injected at injector-creation
     * time only.
     *
     * @return a possibly empty set
     */
    Set<InjectionPoint> getInjectionPoints();

}