package com.google.inject;

import junit.framework.TestCase;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicReference;

public class BindingOrderTest extends TestCase {

    public void testBindingOutOfOrder() {
        Guice.createInjector(new AbstractModule() {
            protected void configure() {
                bind(BoundFirst.class);
                bind(BoundSecond.class).to(BoundSecondImpl.class);
            }
        });
    }

    public void testBindingOrderAndScopes() {
        Injector injector = Guice.createInjector(new AbstractModule() {
            protected void configure() {
                bind(A.class);
                bind(B.class).asEagerSingleton();
            }
        });

        assertSame(injector.getInstance(A.class).b, injector.getInstance(A.class).b);
    }

    public void testBindingWithExtraThreads() throws InterruptedException {
        final CountDownLatch ready = new CountDownLatch(1);
        final CountDownLatch done = new CountDownLatch(1);
        final AtomicReference<B> ref = new AtomicReference<B>();

        final Object createsAThread = new Object() {
            @Inject
            void createAnotherThread(final Injector injector) {
                new Thread() {
                    public void run() {
                        ready.countDown();
                        A a = injector.getInstance(A.class);
                        ref.set(a.b);
                        done.countDown();
                    }
                }.start();

                // to encourage collisions, we make sure the other thread is running before returning
                try {
                    ready.await();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            }
        };

        Guice.createInjector(new AbstractModule() {
            protected void configure() {
                requestInjection(createsAThread);
                bind(A.class).toInstance(new A());
            }
        });

        done.await();
        assertNotNull(ref.get());
    }

    interface BoundSecond {
    }

    public static class BoundFirst {
        @Inject
        public BoundFirst(BoundSecond boundSecond) {
        }
    }

    static class BoundSecondImpl implements BoundSecond {
    }

    static class A {
        @Inject
        B b;
    }

    static class B {
    }
}
