package com.google.inject.example;

import static junit.framework.Assert.assertTrue;

public class ClientServiceWithDependencyInjection {

    public void testClient() {
        MockService mock = new MockService();
        Client client = new Client(mock);
        client.go();
        assertTrue(mock.isGone());
    }

    public interface Service {
        void go();
    }

    public static class ServiceImpl implements ClientServiceWithDependencyInjection.Service {
        public void go() {
            // ...
        }
    }

    public static class ServiceFactory {

        private static final Service service = new ServiceImpl();

        private ServiceFactory() {
        }

        public static Service getInstance() {
            return service;
        }
    }

    public static class Client {

        private final Service service;

        public Client(Service service) {
            this.service = service;
        }

        public void go() {
            service.go();
        }
    }

    public static class ClientFactory {

        private ClientFactory() {
        }

        public static Client getInstance() {
            Service service = ServiceFactory.getInstance();
            return new Client(service);
        }
    }

    public static class MockService implements Service {

        private boolean gone = false;

        public void go() {
            gone = true;
        }

        public boolean isGone() {
            return gone;
        }
    }
}
