package com.google.inject.example;

import com.google.inject.CreationException;
import com.google.inject.Guice;
import com.google.inject.ImplementedBy;
import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Singleton;
import junit.framework.Assert;

public class ClientServiceWithGuiceDefaults {

// 44 lines

    public static void main(String[] args) throws CreationException {
        new ClientServiceWithGuiceDefaults().testClient();
        Injector injector = Guice.createInjector();
        Client client = injector.getProvider(Client.class).get();
    }

    public void testClient() {
        MockService mock = new MockService();
        Client client = new Client(mock);
        client.go();
        Assert.assertTrue(mock.isGone());
    }

    @ImplementedBy(ServiceImpl.class)
    public interface Service {
        void go();
    }

    @Singleton
    public static class ServiceImpl implements ClientServiceWithGuiceDefaults.Service {
        public void go() {
            // ...
        }
    }

    public static class Client {

        private final Service service;

        @Inject
        public Client(Service service) {
            this.service = service;
        }

        public void go() {
            service.go();
        }
    }

    public static class MockService implements Service {

        private boolean gone = false;

        public void go() {
            gone = true;
        }

        public boolean isGone() {
            return gone;
        }
    }
}
